@@ -1,4 +1,4 @@
-# Copyright (c) 2016-2023 Martin Donath <martin.donath@squidfunk.com>
+# Copyright (c) 2016-2024 Martin Donath <martin.donath@squidfunk.com>
 
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to
@@ -18,14 +18,17 @@
 # FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 # IN THE SOFTWARE.
 
+from __future__ import annotations
+
 import logging
 import os
 import posixpath
-import readtime
 import yaml
 
-from babel.dates import format_date
+from babel.dates import format_date, format_datetime
 from datetime import datetime
+from jinja2 import pass_context
+from jinja2.runtime import Context
 from mkdocs.config.defaults import MkDocsConfig
 from mkdocs.exceptions import PluginError
 from mkdocs.plugins import BasePlugin, event_priority
@@ -33,16 +36,17 @@ from mkdocs.structure import StructureItem
 from mkdocs.structure.files import File, Files, InclusionLevel
 from mkdocs.structure.nav import Navigation, Section
 from mkdocs.structure.pages import Page
-from mkdocs.utils import get_relative_url
+from mkdocs.utils import copy_file, get_relative_url
+from mkdocs.utils.templates import url_filter
 from paginate import Page as Pagination
 from shutil import rmtree
 from tempfile import mkdtemp
 from yaml import SafeLoader
 
-from .author import Author, Authors
+from .author import Authors
 from .config import BlogConfig
+from .readtime import readtime
 from .structure import Archive, Category, Excerpt, Post, View
-from .templates import url_filter
 
 # -----------------------------------------------------------------------------
 # Classes
@@ -60,7 +64,7 @@ class BlogPlugin(BasePlugin[BlogConfig]):
         self.is_serve = False
         self.is_dirty = False
 
-        # Initialize a temporary directory
+        # Initialize temporary directory
         self.temp_dir = mkdtemp()
 
     # Determine whether we're serving the site
@@ -87,12 +91,18 @@ class BlogPlugin(BasePlugin[BlogConfig]):
             self.config.categories_toc = self.config.blog_toc
 
         # By default, drafts are rendered when the documentation is served,
-        # but not when it is built, for a better authoring experience
+        # but not when it is built, for a better user experience
         if self.is_serve and self.config.draft_on_serve:
             self.config.draft = True
 
-    # Remove posts before constructing navigation (run later) - allow other
-    # plugins to alter the list of files and navigation prior to this plugin
+    # Resolve and load posts and generate views (run later) - we want to allow
+    # other plugins to add generated posts or views, so we run this plugin as
+    # late as possible. We also need to remove the posts from the navigation
+    # before navigation is constructed, as the entrypoint should be considered
+    # to be the active page for each post. The URLs of posts are computed before
+    # Markdown processing, so that when linking to and from posts, behavior is
+    # exactly the same as with regular documentation pages. We create all pages
+    # related to posts as part of this plugin, so we control the entire process.
     @event_priority(-50)
     def on_files(self, files, *, config):
         if not self.config.enabled:
@@ -102,84 +112,106 @@ class BlogPlugin(BasePlugin[BlogConfig]):
         root = posixpath.normpath(self.config.blog_dir)
         site = config.site_dir
 
-        # Compute path to posts directory
+        # Compute and normalize path to posts directory
         path = self.config.post_dir.format(blog = root)
         path = posixpath.normpath(path)
 
-        # Temporarily remove posts and adjust destination paths for assets
-        for file in files:
+        # Adjust destination paths for media files
+        for file in files.media_files():
             if not file.src_uri.startswith(path):
                 continue
 
-            # We must exclude all files related to posts from here on, so MkDocs
-            # will not attach the posts to the navigation when auto-populating.
-            # We add them back in `on_nav`, so MkDocs processes them, unless
-            # excluded by being tagged as a draft or through other means.
-            if file.is_documentation_page():
-                file.inclusion = InclusionLevel.EXCLUDED
-
-            # We also need to adjust destination paths for assets to remove the
+            # We need to adjust destination paths for assets to remove the
             # purely functional posts directory prefix when building
-            if file.is_media_file():
-                file.dest_uri      = file.dest_uri.replace(path, root)
-                file.abs_dest_path = os.path.join(site, file.dest_path)
-                file.url           = file.url.replace(path, root)
-
-        # Generate entrypoint, if it does not exist yet
-        self._generate(config, files)
-
-    # Resolve and load posts and generate indexes (run later) - we resolve all
-    # posts after the navigation is constructed in order to allow other plugins
-    # to alter the navigation (e.g. awesome-pages) before we start to add pages
-    # generated by this plugin. Post URLs must be computed before any Markdown
-    # processing, so that when linking to and from posts, MkDocs behaves exactly
-    # the same as with regular documentation pages. We create all pages related
-    # to posts as part of this plugin, so we control the entire process.
-    @event_priority(-50)
-    def on_nav(self, nav, *, config, files):
-        if not self.config.enabled:
-            return
+            file.dest_uri      = file.dest_uri.replace(path, root)
+            file.abs_dest_path = os.path.join(site, file.dest_path)
+            file.url           = file.url.replace(path, root)
 
         # Resolve entrypoint and posts sorted by descending date - if the posts
         # directory or entrypoint do not exist, they are automatically created
-        self.blog = self._resolve(files, config, nav)
+        self.blog = self._resolve(files, config)
         self.blog.posts = sorted(
             self._resolve_posts(files, config),
             key = lambda post: post.config.date.created,
             reverse = True
         )
 
+        # Generate views for archive
+        if self.config.archive:
+            self.blog.views.extend(
+                self._generate_archive(config, files)
+            )
+
+        # Generate views for categories
+        if self.config.categories:
+            self.blog.views.extend(sorted(
+                self._generate_categories(config, files),
+                key = lambda view: view.name,
+                reverse = False
+            ))
+
+        # Generate pages for views
+        if self.config.pagination:
+            for view in self._resolve_views(self.blog):
+                for page in self._generate_pages(view, config, files):
+                    view.pages.append(page)
+
+        # Ensure that entrypoint is always included in navigation
+        self.blog.file.inclusion = InclusionLevel.INCLUDED
+
+    # Attach posts and views to navigation (run later) - again, we allow other
+    # plugins to alter the navigation before we start to attach posts and views
+    # generated by this plugin at the correct locations in the navigation. Also,
+    # we make sure to correct links to the parent and siblings of each page.
+    @event_priority(-50)
+    def on_nav(self, nav, *, config, files):
+        if not self.config.enabled:
+            return
+
+        # If we're not building a standalone blog, the entrypoint will always
+        # have a parent when it is included in the navigation. The parent is
+        # essential to correctly resolve the location where the archive and
+        # category views are attached. If the entrypoint doesn't have a parent,
+        # we know that the author did not include it in the navigation, so we
+        # explicitly mark it as not included.
+        if not self.blog.parent and self.config.blog_dir != ".":
+            self.blog.file.inclusion = InclusionLevel.NOT_IN_NAV
+
         # Attach posts to entrypoint without adding them to the navigation, so
-        # that the entrypoint is considered to be the active page for each post.
-        # Hack: MkDocs has a bug where pages that are marked to be not in the
-        # navigation are auto-populated nonetheless - see https://t.ly/7aYnO
+        # that the entrypoint is considered to be the active page for each post
         self._attach(self.blog, [None, *reversed(self.blog.posts), None])
         for post in self.blog.posts:
             post.file.inclusion = InclusionLevel.NOT_IN_NAV
 
-        # Generate and attach views for archive
+        # Revert temporary exclusion of views from navigation
+        for view in self._resolve_views(self.blog):
+            view.file.inclusion = self.blog.file.inclusion
+            for page in view.pages:
+                page.file.inclusion = self.blog.file.inclusion
+
+        # Attach views for archive
         if self.config.archive:
-            views = [*self._generate_archive(config, files)]
-            self.blog.views.extend(views)
+            title = self._translate(self.config.archive_name, config)
+            views = [_ for _ in self.blog.views if isinstance(_, Archive)]
 
             # Attach and link views for archive
-            title = self._translate(self.config.archive_name, config)
-            self._attach_to(self.blog, Section(title, views), nav)
+            if self.blog.file.inclusion.is_in_nav():
+                self._attach_to(self.blog, Section(title, views), nav)
 
-        # Generate and attach views for categories
+        # Attach views for categories
         if self.config.categories:
-            views = [*self._generate_categories(config, files)]
-            self.blog.views.extend(views)
-
-            # Attach and link views for categories
             title = self._translate(self.config.categories_name, config)
-            self._attach_to(self.blog, Section(title, views), nav)
+            views = [_ for _ in self.blog.views if isinstance(_, Category)]
 
-        # Paginate generated views, if enabled
+            # Attach and link views for categories, if any
+            if self.blog.file.inclusion.is_in_nav() and views:
+                self._attach_to(self.blog, Section(title, views), nav)
+
+        # Attach pages for views
         if self.config.pagination:
-            for view in [*self._resolve_views(self.blog)]:
-                for page in self._generate_pages(view, config, files):
-                    view.pages.append(page)
+            for view in self._resolve_views(self.blog):
+                for at in range(1, len(view.pages)):
+                    self._attach_at(view.parent, view, view.pages[at])
 
     # Prepare post for rendering (run later) - allow other plugins to alter
     # the contents or metadata of a post before it is rendered and make sure
@@ -197,18 +229,18 @@ class BlogPlugin(BasePlugin[BlogConfig]):
                 return
 
             # We set the contents of the view to its title if pagination should
-            # not keep the content of the original view on paginaged views
+            # not keep the content of the original view on paginated views
             if not self.config.pagination_keep_content:
-                if page in self._resolve_views(self.blog):
-                    assert isinstance(page, View)
-                    if 0 < page.pages.index(page):
-                        main = page.parent
+                view = self._resolve_original(page)
+                if view in self._resolve_views(self.blog):
 
-                        # We need to use the rendered title of the original view
-                        # if the author set the title in the page's contents, or
-                        # it would be overridden with the one set in mkdocs.yml,
-                        # which would result in inconsistent headings
-                        name = main._title_from_render or main.title
+                    # If the current view is paginated, use the rendered title
+                    # of the original view in case the author set the title in
+                    # the page's contents, or it would be overridden with the
+                    # one set in mkdocs.yml, leading to inconsistent headings
+                    assert isinstance(view, View)
+                    if view != page:
+                        name = view._title_from_render or view.title
                         return f"# {name}"
 
             # Nothing more to be done for views
@@ -223,17 +255,6 @@ class BlogPlugin(BasePlugin[BlogConfig]):
                 # Append to list of authors
                 page.authors.append(self.authors[name])
 
-        # Compute readtime of post, if enabled and not explicitly set
-        if self.config.post_readtime:
-            rate = self.config.post_readtime_words_per_minute
-
-            # There's a bug in the readtime library which causes it to fail if
-            # the input string contains emojis - see https://t.ly/qEoHq
-            if not page.config.readtime:
-                data = markdown.encode("unicode_escape")
-                read = readtime.of_markdown(data, rate)
-                page.config.readtime = read.minutes
-
         # Extract settings for excerpts
         separator      = self.config.post_excerpt_separator
         max_authors    = self.config.post_excerpt_max_authors
@@ -244,13 +265,12 @@ class BlogPlugin(BasePlugin[BlogConfig]):
         # is not already present, so we can remove footnotes or other content
         # from the excerpt without affecting the content of the excerpt
         if separator not in page.markdown:
-            path = page.file.src_path
             if self.config.post_excerpt == "required":
+                docs = os.path.relpath(config.docs_dir)
+                path = os.path.relpath(page.file.abs_src_path, docs)
                 raise PluginError(
-                    f"Couldn't find '{separator}' separator in '{path}'"
+                    f"Couldn't find '{separator}' in post '{path}' in '{docs}'"
                 )
-            else:
-                page.markdown += f"\n\n{separator}"
 
         # Create excerpt for post and inherit authors and categories - excerpts
         # can contain a subset of the authors and categories of the post
@@ -258,6 +278,22 @@ class BlogPlugin(BasePlugin[BlogConfig]):
         page.excerpt.authors    = page.authors[:max_authors]
         page.excerpt.categories = page.categories[:max_categories]
 
+    # Process posts
+    def on_page_content(self, html, *, page, config, files):
+        if not self.config.enabled:
+            return
+
+        # Skip if page is not a post managed by this instance - this plugin has
+        # support for multiple instances, which is why this check is necessary
+        if page not in self.blog.posts:
+            return
+
+        # Compute readtime of post, if enabled and not explicitly set
+        if self.config.post_readtime:
+            words_per_minute = self.config.post_readtime_words_per_minute
+            if not page.config.readtime:
+                page.config.readtime = readtime(html, words_per_minute)
+
     # Register template filters for plugin
     def on_env(self, env, *, config, files):
         if not self.config.enabled:
@@ -267,9 +303,25 @@ class BlogPlugin(BasePlugin[BlogConfig]):
         def date_filter(date: datetime):
             return self._format_date_for_post(date, config)
 
+        # Patch URL template filter to add support for paginated views, i.e.,
+        # that paginated views never link to themselves but to the main view
+        @pass_context
+        def url_filter_with_pagination(context: Context, url: str | None):
+            page = context["page"]
+
+            # If the current page is a view, check if the URL links to the page
+            # itself, and replace it with the URL of the main view
+            if isinstance(page, View):
+                view = self._resolve_original(page)
+                if page.url == url:
+                    url = view.url
+
+            # Forward to original template filter
+            return url_filter(context, url)
+
         # Register custom template filters
         env.filters["date"] = date_filter
-        env.filters["url"]  = url_filter
+        env.filters["url"]  = url_filter_with_pagination
 
     # Prepare view for rendering (run latest) - views are rendered last, as we
     # need to mutate the navigation to account for pagination. The main problem
@@ -282,27 +334,10 @@ class BlogPlugin(BasePlugin[BlogConfig]):
 
         # Skip if page is not a view managed by this instance - this plugin has
         # support for multiple instances, which is why this check is necessary
-        if page not in self._resolve_views(self.blog):
+        view = self._resolve_original(page)
+        if view not in self._resolve_views(self.blog):
             return
 
-        # Retrieve parent view or section
-        assert isinstance(page, View)
-        main = page.parent
-
-        # If this page is a view, and the parent page is a view as well, we got
-        # a paginated view and need to replace the parent with the current view.
-        # Paginated views are always rendered at the end of the build, which is
-        # why we can safely mutate the navigation at this point
-        if isinstance(main, View):
-            page.parent = main.parent
-
-            # Replace view in navigation and rewire it - the current view in the
-            # navigation becomes the main view, thus the entire chain moves one
-            # level up. It's essential that the rendering order is linear, or
-            # else we might end up with a broken navigation.
-            items = self._resolve_siblings(main, nav)
-            items[items.index(main)] = page
-
         # Render excerpts and prepare pagination
         posts, pagination = self._render(page)
 
@@ -344,28 +379,24 @@ class BlogPlugin(BasePlugin[BlogConfig]):
 
     # Resolve entrypoint - the entrypoint of the blog must have been created
     # if it did not exist before, and hosts all posts sorted by descending date
-    def _resolve(self, files: Files, config: MkDocsConfig, nav: Navigation):
+    def _resolve(self, files: Files, config: MkDocsConfig):
         path = os.path.join(self.config.blog_dir, "index.md")
         path = os.path.normpath(path)
 
-        # Obtain entrypoint page
+        # Create entrypoint, if it does not exist - note that the entrypoint is
+        # created in the docs directory, not in the temporary directory
+        docs = os.path.relpath(config.docs_dir)
+        name = os.path.join(docs, path)
+        if not os.path.isfile(name):
+            file = self._path_to_file(path, config, temp = False)
+            files.append(file)
+
+            # Create file in docs directory
+            self._save_to_file(file.abs_src_path, "# Blog\n\n")
+
+        # Create and return entrypoint
         file = files.get_file_from_path(path)
-        page = file.page
-
-        # Create entrypoint view and attach to parent
-        view = View(page.title, file, config)
-        self._attach(page.parent, [
-            page.previous_page,
-            view,
-            page.next_page
-        ])
-
-        # Update entrypoint in navigation
-        for items in [self._resolve_siblings(view, nav), nav.pages]:
-            items[items.index(page)] = view
-
-        # Return view
-        return view
+        return View(None, file, config)
 
     # Resolve post - the caller must make sure that the given file points to an
     # actual post (and not a page), or behavior might be unpredictable
@@ -397,13 +428,14 @@ class BlogPlugin(BasePlugin[BlogConfig]):
         if not os.path.isdir(name):
             os.makedirs(name, exist_ok = True)
 
-        # Filter posts from pages - prior to calling this function, the caller
-        # should've excluded all posts, so they're not listed in the navigation
-        inclusion = InclusionLevel.is_excluded
-        for file in files.documentation_pages(inclusion = inclusion):
+        # Filter posts from pages
+        for file in files.documentation_pages():
             if not file.src_path.startswith(path):
                 continue
 
+            # Temporarily remove post from navigation
+            file.inclusion = InclusionLevel.EXCLUDED
+
             # Resolve post - in order to determine whether a post should be
             # excluded, we must load it and analyze its metadata. All posts
             # marked as drafts are excluded, except for when the author has
@@ -428,13 +460,13 @@ class BlogPlugin(BasePlugin[BlogConfig]):
             return config.authors
 
         # Open file and parse as YAML
-        with open(file, encoding = "utf-8") as f:
+        with open(file, encoding = "utf-8-sig") as f:
             config.config_file_path = os.path.abspath(file)
             try:
                 config.load_dict(yaml.load(f, SafeLoader) or {})
 
             # The authors file could not be loaded because of a syntax error,
-            # which we display to the user with a nice error message
+            # which we display to the author with a nice error message
             except Exception as e:
                 raise PluginError(
                     f"Error reading authors file '{path}' in '{docs}':\n"
@@ -443,19 +475,6 @@ class BlogPlugin(BasePlugin[BlogConfig]):
 
         # Validate authors and throw if errors occurred
         errors, warnings = config.validate()
-        if not config.authors and warnings:
-            log.warning(
-                f"Action required: the format of the authors file changed.\n"
-                f"All authors must now be located under the 'authors' key.\n"
-                f"Please adjust '{file}' to match:\n"
-                f"\n"
-                f"authors:\n"
-                f"  squidfunk:\n"
-                f"    avatar: https://avatars.githubusercontent.com/u/932156\n"
-                f"    description: Creator\n"
-                f"    name: Martin Donath\n"
-                f"\n"
-            )
         for _, w in warnings:
             log.warning(w)
         for _, e in errors:
@@ -467,8 +486,7 @@ class BlogPlugin(BasePlugin[BlogConfig]):
         # Return authors
         return config.authors
 
-    # Resolve views and pages of the given view that were generated by this
-    # plugin when building the site and yield them in pre-order
+    # Resolve views of the given view in pre-order
     def _resolve_views(self, view: View):
         yield view
 
@@ -478,11 +496,6 @@ class BlogPlugin(BasePlugin[BlogConfig]):
                 assert isinstance(next, View)
                 yield next
 
-        # Resolve pages
-        for page in view.pages:
-            assert isinstance(page, View)
-            yield page
-
     # Resolve siblings of a navigation item
     def _resolve_siblings(self, item: StructureItem, nav: Navigation):
         if isinstance(item.parent, Section):
@@ -490,58 +503,15 @@ class BlogPlugin(BasePlugin[BlogConfig]):
         else:
             return nav.items
 
-    # -------------------------------------------------------------------------
-
-    # Attach a list of pages to each other and to the given parent item without
-    # explicitly adding them to the navigation, which can be done by the caller
-    def _attach(self, parent: StructureItem, pages: list[Page]):
-        for tail, page, head in zip(pages, pages[1:], pages[2:]):
-
-            # Link page to parent and siblings
-            page.parent        = parent
-            page.previous_page = tail
-            page.next_page     = head
-
-    # Attach a section as a sibling to the given view, make sure it's pages are
-    # part of the navigation, and ensure all pages are linked correctly
-    def _attach_to(self, view: View, section: Section, nav: Navigation):
-        section.parent = view.parent
-
-        # Resolve siblings, which are the children of the parent section, or
-        # the top-level list of navigation items if the view is at the root of
-        # the project, and append the given section to it. It's currently not
-        # possible to chose the position of a section.
-        items = self._resolve_siblings(view, nav)
-        items.append(section)
-
-        # Find last sibling that is a page, skipping sections, as we need to
-        # append the given section after all other pages
-        tail = next(item for item in reversed(items) if isinstance(item, Page))
-        head = tail.next_page
-
-        # Attach section to navigation and pages to each other
-        nav.pages.extend(section.children)
-        self._attach(section, [tail, *section.children, head])
+    # Resolve original page or view (e.g. for paginated views)
+    def _resolve_original(self, page: Page):
+        if isinstance(page, View) and page.pages:
+            return page.pages[0]
+        else:
+            return page
 
     # -------------------------------------------------------------------------
 
-    # Generate entrypoint - the entrypoint must always be present, and thus is
-    # created before the navigation is constructed if it does not exist yet
-    def _generate(self, config: MkDocsConfig, files: Files):
-        path = os.path.join(self.config.blog_dir, "index.md")
-        path = os.path.normpath(path)
-
-        # Create entrypoint, if it does not exist - note that the entrypoint is
-        # added to the docs directory, not to the temporary directory
-        docs = os.path.relpath(config.docs_dir)
-        file = os.path.join(docs, path)
-        if not os.path.isfile(file):
-            file = self._path_to_file(path, config, temp = False)
-            self._save_to_file(file.abs_src_path, "# Blog\n\n")
-
-            # Append entrypoint to files
-            files.append(file)
-
     # Generate views for archive - analyze posts and generate the necessary
     # views, taking the date format provided by the author into account
     def _generate_archive(self, config: MkDocsConfig, files: Files):
@@ -552,16 +522,21 @@ class BlogPlugin(BasePlugin[BlogConfig]):
             name = self._format_date_for_archive(date, config)
             path = self._format_path_for_archive(post, config)
 
-            # Create view for archive if it doesn't exist
+            # Create file for view, if it does not exist
             file = files.get_file_from_path(path)
-            if not file:
+            if not file or self.temp_dir not in file.abs_src_path:
                 file = self._path_to_file(path, config)
-                self._save_to_file(file.abs_src_path, f"# {name}")
-
-                # Create and yield archive view
-                yield Archive(name, file, config)
                 files.append(file)
 
+                # Create file in temporary directory and temporarily remove
+                # from navigation, as we'll add it at a specific location
+                self._save_to_file(file.abs_src_path, f"# {name}")
+                file.inclusion = InclusionLevel.EXCLUDED
+
+            # Create and yield view
+            if not isinstance(file.page, Archive):
+                yield Archive(name, file, config)
+
             # Assign post to archive
             assert isinstance(file.page, Archive)
             file.page.posts.append(post)
@@ -583,16 +558,21 @@ class BlogPlugin(BasePlugin[BlogConfig]):
                         f"'{docs}': category '{name}' not in allow list"
                     )
 
-                # Create view for category if it doesn't exist
+                # Create file for view, if it does not exist
                 file = files.get_file_from_path(path)
-                if not file:
+                if not file or self.temp_dir not in file.abs_src_path:
                     file = self._path_to_file(path, config)
-                    self._save_to_file(file.abs_src_path, f"# {name}")
-
-                    # Create and yield category view
-                    yield Category(name, file, config)
                     files.append(file)
 
+                    # Create file in temporary directory and temporarily remove
+                    # from navigation, as we'll add it at a specific location
+                    self._save_to_file(file.abs_src_path, f"# {name}")
+                    file.inclusion = InclusionLevel.EXCLUDED
+
+                # Create and yield view
+                if not isinstance(file.page, Category):
+                    yield Category(name, file, config)
+
                 # Assign post to category and vice versa
                 assert isinstance(file.page, Category)
                 file.page.posts.append(post)
@@ -603,39 +583,76 @@ class BlogPlugin(BasePlugin[BlogConfig]):
     def _generate_pages(self, view: View, config: MkDocsConfig, files: Files):
         yield view
 
-        # Extract settings for pagination
-        step = self.config.pagination_per_page
-        prev = view
-
         # Compute pagination boundaries and create pages - pages are internally
         # handled as copies of a view, as they map to the same source location
+        step = self.config.pagination_per_page
         for at in range(step, len(view.posts), step):
-            base, _ = posixpath.splitext(view.file.src_uri)
+            path = self._format_path_for_pagination(view, 1 + at // step)
 
-            # Compute path and create a file for pagination
-            path = self._format_path_for_pagination(base, 1 + at // step)
-            file = self._path_to_file(path, config)
+            # Create file for view, if it does not exist
+            file = files.get_file_from_path(path)
+            if not file or self.temp_dir not in file.abs_src_path:
+                file = self._path_to_file(path, config)
+                files.append(file)
 
-            # Replace source file system path and append to files
-            file.src_uri      = view.file.src_uri
-            file.abs_src_path = view.file.abs_src_path
-            files.append(file)
+                # Copy file to temporary directory  and temporarily remove
+                # from navigation, as we'll add it at a specific location
+                copy_file(view.file.abs_src_path, file.abs_src_path)
+                file.inclusion = InclusionLevel.EXCLUDED
 
-            # Create view and attach to previous page
-            next = View(view.title, file, config)
-            self._attach(prev, [
-                view.previous_page,
-                next,
-                view.next_page
-            ])
+            # Create and yield view
+            if not isinstance(file.page, View):
+                yield view.__class__(None, file, config)
 
-            # Assign posts and pages to view
-            next.posts = view.posts
-            next.pages = view.pages
+            # Assign pages and posts to view
+            assert isinstance(file.page, View)
+            file.page.pages = view.pages
+            file.page.posts = view.posts
 
-            # Continue with next page
-            prev = next
-            yield next
+    # -------------------------------------------------------------------------
+
+    # Attach a list of pages to each other and to the given parent item without
+    # explicitly adding them to the navigation, which can be done by the caller
+    def _attach(self, parent: StructureItem, pages: list[Page]):
+        for tail, page, head in zip(pages, pages[1:], pages[2:]):
+
+            # Link page to parent and siblings
+            page.parent        = parent
+            page.previous_page = tail
+            page.next_page     = head
+
+            # If the page is a view, we know that we generated it and need to
+            # link its siblings back to the view
+            if isinstance(page, View):
+                view = self._resolve_original(page)
+                if tail: tail.next_page     = view
+                if head: head.previous_page = view
+
+    # Attach a page to the given parent and link it to the previous and next
+    # page of the given host - this is exclusively used for paginated views
+    def _attach_at(self, parent: StructureItem, host: Page, page: Page):
+        self._attach(parent, [host.previous_page, page, host.next_page])
+
+    # Attach a section as a sibling to the given view, make sure its pages are
+    # part of the navigation, and ensure all pages are linked correctly
+    def _attach_to(self, view: View, section: Section, nav: Navigation):
+        section.parent = view.parent
+
+        # Resolve siblings, which are the children of the parent section, or
+        # the top-level list of navigation items if the view is at the root of
+        # the project, and append the given section to it. It's currently not
+        # possible to chose the position of a section.
+        items = self._resolve_siblings(view, nav)
+        items.append(section)
+
+        # Find last sibling that is a page, skipping sections, as we need to
+        # append the given section after all other pages
+        tail = next(item for item in reversed(items) if isinstance(item, Page))
+        head = tail.next_page
+
+        # Attach section to navigation and pages to each other
+        nav.pages.extend(section.children)
+        self._attach(section, [tail, *section.children, head])
 
     # -------------------------------------------------------------------------
 
@@ -743,21 +760,36 @@ class BlogPlugin(BasePlugin[BlogConfig]):
         return posixpath.join(self.config.blog_dir, f"{path}.md")
 
     # Format path for pagination
-    def _format_path_for_pagination(self, base: str, page: int):
+    def _format_path_for_pagination(self, view: View, page: int):
         path = self.config.pagination_url_format.format(
             page = page
         )
 
+        # Compute base path for pagination - if the given view is an index file,
+        # we need to pop the file name from the base so it's not part of the URL
+        # and we need to append `index` to the path, so the paginated view is
+        # also an index page - see https://t.ly/71MKF
+        base, _ = posixpath.splitext(view.file.src_uri)
+        if view.is_index:
+            base = posixpath.dirname(base)
+            path = posixpath.join(path, "index")
+
         # Normalize path and strip slashes at the beginning and end
         path = posixpath.normpath(path.strip("/"))
         return posixpath.join(base, f"{path}.md")
 
     # -------------------------------------------------------------------------
 
-    # Format date
+    # Format date - if the given format string refers to a predefined format,
+    # we format the date without a time component in order to keep sane default
+    # behavior, since authors will not expect time to be relevant for most posts
+    # as by our assumptions - see https://t.ly/Yi7ZC
     def _format_date(self, date: datetime, format: str, config: MkDocsConfig):
-        locale = config.theme["language"]
-        return format_date(date, format = format, locale = locale)
+        locale: str = config.theme["language"].replace("-", "_")
+        if format in ["full", "long", "medium", "short"]:
+            return format_date(date, format = format, locale = locale)
+        else:
+            return format_datetime(date, format = format, locale = locale)
 
     # Format date for post
     def _format_date_for_post(self, date: datetime, config: MkDocsConfig):
@@ -814,10 +846,10 @@ class BlogPlugin(BasePlugin[BlogConfig]):
         # Return file
         return file
 
-    # Write the content to the file located at the given path
+    # Create a file with the given content on disk
     def _save_to_file(self, path: str, content: str):
         os.makedirs(os.path.dirname(path), exist_ok = True)
-        with open(path, "w") as f:
+        with open(path, "w", encoding = "utf-8") as f:
             f.write(content)
 
     # -------------------------------------------------------------------------
