@@ -1,4 +1,4 @@
-# Copyright (c) 2016-2023 Martin Donath <martin.donath@squidfunk.com>
+# Copyright (c) 2016-2024 Martin Donath <martin.donath@squidfunk.com>
 
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to
@@ -18,6 +18,19 @@
 # FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 # IN THE SOFTWARE.
 
+# -----------------------------------------------------------------------------
+# Disclaimer
+# -----------------------------------------------------------------------------
+# Please note: this version of the social plugin is not actively development
+# anymore. Instead, Material for MkDocs Insiders ships a complete rewrite of
+# the plugin which is much more powerful and addresses all shortcomings of
+# this implementation. Additionally, the new social plugin allows to create
+# entirely custom social cards. You can probably imagine, that this was a lot
+# of work to pull off. If you run into problems, or want to have additional
+# functionality, please consider sponsoring the project. You can then use the
+# new version of the plugin immediately.
+# -----------------------------------------------------------------------------
+
 import concurrent.futures
 import functools
 import logging
@@ -29,41 +42,39 @@ import sys
 
 from collections import defaultdict
 from hashlib import md5
+from html import unescape
 from io import BytesIO
 from mkdocs.commands.build import DuplicateFilter
-from mkdocs.config import config_options as opt
-from mkdocs.config.base import Config
+from mkdocs.exceptions import PluginError
 from mkdocs.plugins import BasePlugin
+from mkdocs.utils import write_file
 from shutil import copyfile
-from tempfile import TemporaryFile
-from zipfile import ZipFile
+
+from .config import SocialConfig
+
+try:
+    from PIL import Image, ImageDraw, ImageFont
+except ImportError as e:
+    import_errors = {repr(e)}
+else:
+    import_errors = set()
+
+cairosvg_error: str = ""
 
 try:
     from cairosvg import svg2png
-    from PIL import Image, ImageDraw, ImageFont
-    dependencies = True
-except ImportError:
-    dependencies = False
+except ImportError as e:
+    import_errors.add(repr(e))
+except OSError as e:
+    cairosvg_error = str(e)
+
 
 # -----------------------------------------------------------------------------
-# Class
-# -----------------------------------------------------------------------------
-
-# Social plugin configuration scheme
-class SocialPluginConfig(Config):
-    enabled = opt.Type(bool, default = True)
-    cache_dir = opt.Type(str, default = ".cache/plugin/social")
-
-    # Options for social cards
-    cards = opt.Type(bool, default = True)
-    cards_dir = opt.Type(str, default = "assets/images/social")
-    cards_color = opt.Type(dict, default = dict())
-    cards_font = opt.Optional(opt.Type(str))
-
+# Classes
 # -----------------------------------------------------------------------------
 
 # Social plugin
-class SocialPlugin(BasePlugin[SocialPluginConfig]):
+class SocialPlugin(BasePlugin[SocialConfig]):
 
     def __init__(self):
         self._executor = concurrent.futures.ThreadPoolExecutor(4)
@@ -71,22 +82,48 @@ class SocialPlugin(BasePlugin[SocialPluginConfig]):
     # Retrieve configuration
     def on_config(self, config):
         self.color = colors.get("indigo")
+        self.config.cards = self.config.enabled
         if not self.config.cards:
             return
 
-        # Check if required dependencies are installed
-        if not dependencies:
-            log.error(
-                "Required dependencies of \"social\" plugin not found. "
-                "Install with: pip install pillow cairosvg"
+        # Check dependencies
+        if import_errors:
+            raise PluginError(
+                "Required dependencies of \"social\" plugin not found:\n"
+                + str("\n".join(map(lambda x: "- " + x, import_errors)))
+                + "\n\n--> Install with: pip install \"mkdocs-material[imaging]\""
             )
-            sys.exit(1)
+
+        if cairosvg_error:
+            raise PluginError(
+                "\"cairosvg\" Python module is installed, but it crashed with:\n"
+                + cairosvg_error
+                + "\n\n--> Check out the troubleshooting guide: https://t.ly/MfX6u"
+            )
+
+        # Move color options
+        if self.config.cards_color:
+
+            # Move background color to new option
+            value = self.config.cards_color.get("fill")
+            if value:
+                self.config.cards_layout_options["background_color"] = value
+
+            # Move color to new option
+            value = self.config.cards_color.get("text")
+            if value:
+                self.config.cards_layout_options["color"] = value
+
+        # Move font family to new option
+        if self.config.cards_font:
+            value = self.config.cards_font
+            self.config.cards_layout_options["font_family"] = value
 
         # Check if site URL is defined
         if not config.site_url:
             log.warning(
-                "The \"social\" plugin needs the \"site_url\" configuration "
-                "option to be defined. It will likely not work correctly."
+                "The \"site_url\" option is not set. The cards are generated, "
+                "but not linked, so they won't be visible on social media."
             )
 
         # Ensure presence of cache directory
@@ -99,9 +136,12 @@ class SocialPlugin(BasePlugin[SocialPluginConfig]):
         if "palette" in theme:
             palette = theme["palette"]
 
-            # Use first palette, if multiple are defined
+            # Find first palette that includes primary color definition
             if isinstance(palette, list):
-                palette = palette[0]
+                for p in palette:
+                    if "primary" in p and p["primary"]:
+                        palette = p
+                        break
 
             # Set colors according to palette
             if "primary" in palette and palette["primary"]:
@@ -109,7 +149,11 @@ class SocialPlugin(BasePlugin[SocialPluginConfig]):
                 self.color = colors.get(primary, self.color)
 
         # Retrieve color overrides
-        self.color = { **self.color, **self.config.cards_color }
+        options = self.config.cards_layout_options
+        self.color = {
+            "fill": options.get("background_color", self.color["fill"]),
+            "text": options.get("color", self.color["text"])
+        }
 
         # Retrieve logo and font
         self._resized_logo_promise = self._executor.submit(self._load_resized_logo, config)
@@ -147,7 +191,23 @@ class SocialPlugin(BasePlugin[SocialPluginConfig]):
         if "description" in page.meta:
             description = page.meta["description"]
 
-        # Generate social card if not in cache - TODO: values from mkdocs.yml
+        # Check type of meta title - see https://t.ly/m1Us
+        if not isinstance(title, str):
+            log.error(
+                f"Page meta title of page '{page.file.src_uri}' must be a "
+                f"string, but is of type \"{type(title)}\"."
+            )
+            sys.exit(1)
+
+        # Check type of meta description - see https://t.ly/m1Us
+        if not isinstance(description, str):
+            log.error(
+                f"Page meta description of '{page.file.src_uri}' must be a "
+                f"string, but is of type \"{type(description)}\"."
+            )
+            sys.exit(1)
+
+        # Generate social card if not in cache
         hash = md5("".join([
             site_name,
             str(title),
@@ -165,6 +225,9 @@ class SocialPlugin(BasePlugin[SocialPluginConfig]):
         page.meta["meta"] = meta + self._generate_meta(page, config)
 
     def on_post_build(self, config):
+        if not self.config.cards:
+            return
+
         # Check for exceptions
         for promise in self._image_promises:
             promise.result()
@@ -235,8 +298,9 @@ class SocialPlugin(BasePlugin[SocialPluginConfig]):
         width = size[0]
         lines, words = [], []
 
-        # Remove remnant HTML tags
+        # Remove remnant HTML tags and convert HTML entities
         text = re.sub(r"(<[^>]+>)", "", text)
+        text = unescape(text)
 
         # Retrieve y-offset of textbox to correct for spacing
         yoffset = 0
@@ -252,17 +316,6 @@ class SocialPlugin(BasePlugin[SocialPluginConfig]):
                 lines.append(words)
                 words = [word]
 
-        # # Balance words on last line - TODO: overflows when broken word is too long
-        # if len(lines) > 0:
-        #     prev = len(" ".join(lines[-1]))
-        #     last = len(" ".join(words))#
-
-        #     print(last, prev)
-
-        #     # Heuristic: try to find a good ratio
-        #     if last / prev < 0.6:
-        #         words.insert(0, lines[-1].pop())
-
         # Join words for each line and create image
         lines.append(words)
         lines = [" ".join(line) for line in lines]
@@ -286,9 +339,11 @@ class SocialPlugin(BasePlugin[SocialPluginConfig]):
         file, _ = os.path.splitext(page.file.src_uri)
 
         # Compute page title
-        title = page.meta.get("title", page.title)
-        if not page.is_homepage:
-            title = f"{title} - {config.site_name}"
+        if page.is_homepage:
+            title = config.site_name
+        else:
+            page_title = page.meta.get("title", page.title)
+            title = f"{page_title} - {config.site_name}"
 
         # Compute page description
         description = config.site_description
@@ -340,8 +395,15 @@ class SocialPlugin(BasePlugin[SocialPluginConfig]):
         if "logo" in theme:
             _, extension = os.path.splitext(theme["logo"])
 
-            # Load SVG and convert to PNG
             path = os.path.join(config.docs_dir, theme["logo"])
+
+            # Allow users to put the logo inside their custom_dir (theme["logo"] case)
+            if theme.custom_dir:
+                custom_dir_logo = os.path.join(theme.custom_dir, theme["logo"])
+                if os.path.exists(custom_dir_logo):
+                    path = custom_dir_logo
+
+            # Load SVG and convert to PNG
             if extension == ".svg":
                 return self._load_logo_svg(path)
 
@@ -349,10 +411,11 @@ class SocialPlugin(BasePlugin[SocialPluginConfig]):
             return Image.open(path).convert("RGBA")
 
         # Handle icons
-        logo = "material/library"
-        icon = theme["icon"] or {}
+        icon = theme.get("icon") or {}
         if "logo" in icon and icon["logo"]:
             logo = icon["logo"]
+        else:
+            logo = "material/library"
 
         # Resolve path of package
         base = os.path.abspath(os.path.join(
@@ -360,8 +423,15 @@ class SocialPlugin(BasePlugin[SocialPluginConfig]):
             "../.."
         ))
 
+        path = f"{base}/templates/.icons/{logo}.svg"
+
+        # Allow users to put the logo inside their custom_dir (theme["icon"]["logo"] case)
+        if theme.custom_dir:
+            custom_dir_logo = os.path.join(theme.custom_dir, ".icons", f"{logo}.svg")
+            if os.path.exists(custom_dir_logo):
+                path = custom_dir_logo
+
         # Load icon data and fill with color
-        path = f"{base}/.icons/{logo}.svg"
         return self._load_logo_svg(path, self.color["text"])
 
     # Load SVG file and convert to PNG
@@ -377,52 +447,102 @@ class SocialPlugin(BasePlugin[SocialPluginConfig]):
         svg2png(bytestring = data, write_to = file, scale = 10)
         return Image.open(file)
 
-    # Retrieve font
+    # Retrieve font either from the card layout option or from the Material
+    # font defintion. If no font is defined for Material or font is False
+    # then choose a default.
     def _load_font(self, config):
-        name = self.config.cards_font
+        name = self.config.cards_layout_options.get("font_family")
         if not name:
-
-            # Retrieve from theme (default: Roboto)
-            theme = config.theme
-            if theme["font"]:
-                name = theme["font"]["text"]
-            else:
+            material_name = config.theme.get("font", False)
+            if material_name is False:
                 name = "Roboto"
+            else:
+                name = material_name.get("text", "Roboto")
 
-        # Retrieve font files, if not already done
-        files = os.listdir(self.cache)
-        files = [file for file in files if file.endswith(".ttf") or file.endswith(".otf")] or (
-            self._load_font_from_google(name)
-        )
-
-        # Map available font weights to file paths
-        font = dict()
-        for file in files:
-            match = re.search(r"-(\w+)\.[ot]tf$", file)
-            if match:
-                font[match.group(1)] = os.path.join(self.cache, file)
+        # Resolve relevant fonts
+        font = {}
+        for style in ["Regular", "Bold"]:
+            font[style] = self._resolve_font(name, style)
 
         # Return available font weights with fallback
         return defaultdict(lambda: font["Regular"], font)
 
-    # Retrieve font from Google Fonts
-    def _load_font_from_google(self, name):
-        url = "https://fonts.google.com/download?family={}"
-        res = requests.get(url.format(name.replace(" ", "+")), stream = True)
+    # Resolve font family with specific style - if we haven't already done it,
+    # the font family is first downloaded from Google Fonts and the styles are
+    # saved to the cache directory. If the font cannot be resolved, the plugin
+    # must abort with an error.
+    def _resolve_font(self, family: str, style: str):
+        path = os.path.join(self.config.cache_dir, "fonts", family)
 
-        # Write archive to temporary file
-        tmp = TemporaryFile()
-        for chunk in res.iter_content(chunk_size = 32768):
-            tmp.write(chunk)
+        # Fetch font family, if it hasn't been fetched yet
+        if not os.path.isdir(path):
+            self._fetch_font_from_google_fonts(family)
 
-        # Unzip fonts from temporary file
-        zip = ZipFile(tmp)
-        files = [file for file in zip.namelist() if file.endswith(".ttf") or file.endswith(".otf")]
-        zip.extractall(self.cache, files)
+        # Check for availability of font style
+        list = sorted(os.listdir(path))
+        for file in list:
+            name, _ = os.path.splitext(file)
+            if name == style:
+                return os.path.join(path, file)
 
-        # Close and delete temporary file
-        tmp.close()
-        return files
+        # Find regular variant of font family - we cannot rely on the fact that
+        # fonts always have a single regular variant - some of them have several
+        # of them, potentially prefixed with "Condensed" etc. For this reason we
+        # use the first font we find if we find no regular one.
+        fallback = ""
+        for file in list:
+            name, _ = os.path.splitext(file)
+
+            # 1. Fallback: use first font
+            if not fallback:
+                fallback = name
+
+            # 2. Fallback: use regular font - use the shortest one, i.e., prefer
+            # "10pt Regular" over "10pt Condensed Regular". This is a heuristic.
+            if "Regular" in name:
+                if not fallback or len(name) < len(fallback):
+                    fallback = name
+
+        # Fall back to regular font (guess if there are multiple)
+        return self._resolve_font(family, fallback)
+
+    # Fetch font family from Google Fonts
+    def _fetch_font_from_google_fonts(self, family: str):
+        path = os.path.join(self.config.cache_dir, "fonts")
+
+        # Download manifest from Google Fonts - Google returns JSON with syntax
+        # errors, so we just treat the response as plain text and parse out all
+        # URLs to font files, as we're going to rename them anyway. This should
+        # be more resilient than trying to correct the JSON syntax.
+        url = f"https://fonts.google.com/download/list?family={family}"
+        res = requests.get(url)
+
+        # Ensure that the download succeeded
+        if res.status_code != 200:
+            raise PluginError(
+                f"Couldn't find font family '{family}' on Google Fonts "
+                f"({res.status_code}: {res.reason})"
+            )
+
+        # Extract font URLs from manifest
+        for match in re.findall(
+            r"\"(https:(?:.*?)\.[ot]tf)\"", str(res.content)
+        ):
+            with requests.get(match) as res:
+                res.raise_for_status()
+
+                # Extract font family name and style using the content in the
+                # response via ByteIO to avoid writing a temp file. Done to fix
+                # problems with passing a NamedTemporaryFile to
+                # ImageFont.truetype() on Windows, see https://t.ly/LiF_k
+                with BytesIO(res.content) as fontdata:
+                    font = ImageFont.truetype(fontdata)
+                    name, style = font.getname()
+                    name = " ".join([name.replace(family, ""), style]).strip()
+                    target = os.path.join(path, family, f"{name}.ttf")
+
+                # write file to cache
+                write_file(res.content, target)
 
 # -----------------------------------------------------------------------------
 # Data
@@ -433,7 +553,7 @@ log = logging.getLogger("mkdocs")
 log.addFilter(DuplicateFilter())
 
 # Color palette
-colors = dict({
+colors = {
     "red":         { "fill": "#ef5552", "text": "#ffffff" },
     "pink":        { "fill": "#e92063", "text": "#ffffff" },
     "purple":      { "fill": "#ab47bd", "text": "#ffffff" },
@@ -455,4 +575,4 @@ colors = dict({
     "blue-grey":   { "fill": "#546d78", "text": "#ffffff" },
     "black":       { "fill": "#000000", "text": "#ffffff" },
     "white":       { "fill": "#ffffff", "text": "#000000" }
-})
+}
